@@ -1,35 +1,63 @@
-"""
-Python bindings to odesk API
-python-odesk version 0.4.1
-(C) 2010-2011 oDesk
-"""
+# Python bindings to oDesk API
+# python-odesk version 0.5
+# (C) 2010-2013 oDesk
 
+import os
 import time
 import urlparse
 import urllib
 import oauth2 as oauth
+import logging
+
+from .config import BASE_URL
 
 
 from odesk.namespaces import Namespace
-from odesk.http import HttpRequest
 
 
 class OAuth(Namespace):
-    """
-    oAuth support in oDesk is in beta currently.
+
+    """Authorization router.
+
+    Has methods for retrieving access tokens and
+    :py:meth:`~odesk.oauth.OAuth.get_info` method for
+    checking you're authorized successfully and ready to work with API.
     """
 
     api_url = 'auth/'
     version = 1
 
-    request_token_url = 'https://www.odesk.com/api/auth/v1/oauth/token/request'
-    authorize_url = 'https://www.odesk.com/services/api/auth'
-    access_token_url = 'https://www.odesk.com/api/auth/v1/oauth/token/access'
+    request_token_url = os.path.join(
+        BASE_URL, 'api/auth/v1/oauth/token/request')
+    authorize_url = os.path.join(BASE_URL, 'services/api/auth')
+    access_token_url = os.path.join(BASE_URL, 'api/auth/v1/oauth/token/access')
 
-    def urlencode(self, url, key, secret, data={}, method='GET'):
+    def get_oauth_params(self, url, key, secret, data=None, method='GET',
+                         to_header=False, to_dict=False):
         """
-        Converts a mapping object to signed url query
+        Converts a mapping object to signed url query.
+
+        *Parameters:*
+          :url:        Target url
+
+          :key:        Public API key
+
+          :secret:     Public API key secret
+
+          :data:       Dictionary with data parameters
+
+          :method:     Mehtod to be called, default is ``GET``
+
+          :to_header:  If ``True``, data will be encoded as auth
+                       headers
+
         """
+        # Temporary not use incoming data, just generate headers
+        if data is None:
+            data = {}
+        else:
+            data = data.copy()
+
         token = oauth.Token(key, secret)
         consumer = self.get_oauth_consumer()
         data.update({
@@ -42,22 +70,27 @@ class OAuth(Namespace):
         request = oauth.Request(method=method, url=url, parameters=data)
         signature_method = oauth.SignatureMethod_HMAC_SHA1()
         request.sign_request(signature_method, consumer, token)
+
+        if to_header:
+            return request.to_header()
+
         return request.to_postdata()
 
     def get_oauth_consumer(self):
         """
-        Returns OAuth consumer object
+        Returns OAuth consumer object.
         """
         return oauth.Consumer(self.client.public_key, self.client.secret_key)
 
     def get_request_token(self):
         """
-        Returns request token and request token secret
+        Returns request token and request token secret.
         """
         client = oauth.Client(self.get_oauth_consumer())
         response, content = client.request(self.request_token_url, 'POST')
         if response.get('status') != '200':
-            raise Exception("Invalid request token response: %s." % content)
+            raise Exception(
+                "Invalid request token response: {0}.".format(content))
         request_token = dict(urlparse.parse_qsl(content))
         self.request_token = request_token.get('oauth_token')
         self.request_token_secret = request_token.get('oauth_token_secret')
@@ -65,7 +98,7 @@ class OAuth(Namespace):
 
     def get_authorize_url(self, callback_url=None):
         """
-        Returns authentication URL to be used in a browser
+        Returns authentication URL to be used in a browser.
         """
         oauth_token = getattr(self, 'request_token', None) or\
             self.get_request_token()[0]
@@ -74,24 +107,37 @@ class OAuth(Namespace):
                 'oauth_callback': callback_url})
         else:
             params = urllib.urlencode({'oauth_token': oauth_token})
-        return '%s?%s' % (self.authorize_url, params)
+        return '{0}?{1}'.format(self.authorize_url, params)
 
     def get_access_token(self, verifier):
         """
-        Returns access token and access token secret
+        Returns access token and access token secret.
         """
         try:
             request_token = self.request_token
             request_token_secret = self.request_token_secret
-        except AttributeError:
+        except AttributeError, e:
+            logger = logging.getLogger('python-odesk')
+            logger.debug(e)
             raise Exception("At first you need to call get_authorize_url")
         token = oauth.Token(request_token, request_token_secret)
         token.set_verifier(verifier)
         client = oauth.Client(self.get_oauth_consumer(), token)
         response, content = client.request(self.access_token_url, 'POST')
         if response.get('status') != '200':
-            raise Exception("Invalid access token response: %s." % content)
+            raise Exception(
+                "Invalid access token response: {0}.".format(content))
         access_token = dict(urlparse.parse_qsl(content))
         self.access_token = access_token.get('oauth_token')
         self.access_token_secret = access_token.get('oauth_token_secret')
         return self.access_token, self.access_token_secret
+
+    def get_info(self):
+        """
+        Get a detailed info about current authnticated user \
+        and some data from his profile.
+
+        """
+        url = 'info'
+        result = self.get(url)
+        return result
